/*
    Copyright (C) 1999 by Joe Pasqua <joe@NoRoomAtTheInn.org>.
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.


    THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

import java.util.Vector;

class Arg {
    private String  type;
    private String  name;
    private Value   min;
    private Value   max;

    Arg(String type, String name, String min, String max)
        throws ParseException
    {
        this.type = type;
        this.name = name;
        this.min = (min == null) ? null : Type.getVal(type, new BasicVal(min));
        this.max = (max == null) ? null : Type.getVal(type, new BasicVal(max));
    }

    Arg(String type, String name)
        throws ParseException
    {
        this(type, name, null, null);
    }

    public String getName() { return name; }

    public String getTypeName() { return type; }

    static Vector/*Value*/ getArgs(Arg[] args, Vector/*BasicVal*/ inputs)
            throws ParseException
    {
        if (args == null && (inputs == null || inputs.size() == 0))
            return new Vector(1);
        if (args.length != inputs.size()) {
            String names = "";
            for (int i = 0; i < args.length; i++) {
                if (i > 0)
                    names += ", ";
                names += args[i].name;
            }
            throw new ParseException(
                    "Wrong number of arguments: found " + inputs.size() +
                    ", expecting " + args.length + " (" + names + ")");
        }

        Vector  vals = new Vector(args.length);
        for (int i = 0; i < args.length; i++) {
            Arg         arg = (Arg)args[i];
            BasicVal    input = (BasicVal)inputs.elementAt(i);

            if (!input.isLiteral) {
                Symbol s = XACompiler.symTab.get(input.value);
                if (s.isVar)
                    throw new ParseException(
                        "Parameter " + (i+1) + " can not be a variable");
            }
            Value v = Type.getVal(arg.type, input);
            if (arg.min != null && v.lt(arg.min))
                throw new ParseException(
                        "Value specified (" + v + ") is less than the " +
                        "min accepatable value (" + arg.min + ")");
            if (arg.max != null && v.gt(arg.max))
                throw new ParseException(
                        "Value specified (" + v + ") is greater than the " +
                        "max accepatable value (" + arg.max + ")");
            vals.addElement(v);
        }

        return vals;
    }

}
