/*
    Copyright (C) 1999 by Joe Pasqua <joe@NoRoomAtTheInn.org>.
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.


    THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

import java.util.Hashtable;
import java.util.Vector;

class NumberType extends Type {

    private int curVarNum = -1;

    NumberType() {
        methods = new Hashtable(1);
        methods.put("compare", new CompareMethod());
        methods.put("set", new SetMethod());
    }

    Value _new(BasicVal v, boolean isVar)
            throws ParseException
    {
        if (isVar && curVarNum > 63)
            throw new ParseException("Exceeded maximum # of Variables");
        NumberVal    sv = (NumberVal)super._new(v, isVar);
        sv.setVarNum(isVar ? ++curVarNum : -1);
        return sv;
    }

    class CompareMethod implements Type.Method {

        public short[] codeGen(Value target, Vector inputs)
                throws ParseException
        {
            Arg[]  argDesc = {
                new Arg("Cond", "relational operator"),
                new Arg("Number", "rhs")
            };
            Vector  args = Arg.getArgs(argDesc, inputs);

            int varNum = ((NumberVal)target).getVarNum();
            if (varNum < 0) {   // Not a Variable!
                throw new ParseException(
                    "LHS of comparison must be a variable");
            }

            return Type.encode(
                0x00, 0xA0 + ((CondVal)args.elementAt(0)).condition(),
                varNum, ((NumberVal)args.elementAt(1)).intValue());
        }
    }

    class SetMethod implements Type.Method {

        public short[] codeGen(Value target, Vector inputs)
                throws ParseException
        {
            Arg[]  argDesc = {
                new Arg("AsnOp", "assignment operator"),
                new Arg("Number", "rhs")
            };
            Vector  args = Arg.getArgs(argDesc, inputs);

            int varNum = ((NumberVal)target).getVarNum();
            if (varNum < 0) {   // Not a Variable!
                throw new ParseException(
                    "LHS of assignment must be a variable");
            }

            return Type.encode(
                0x04, 0xA0 + ((AsnOpVal)args.elementAt(0)).op(),
                varNum, ((NumberVal)args.elementAt(1)).intValue());
        }
    }
}

