/*
    Copyright (C) 1999 by Joe Pasqua <joe@NoRoomAtTheInn.org>.
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.


    THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

import java.util.Hashtable;
import java.util.Vector;

class NumberVal extends Value {
    protected int varNum;
    protected int value;

    int intValue() { return value; }

    void init(Type t, String v)
            throws ParseException
    {
        super.init(t, v);
        try {
            value = (Integer.valueOf(v)).intValue();
            if (value < 0 || value > 65535)
                throw new ParseException(
                    "Value out of range (0-65535): " + v);
        } catch (NumberFormatException e) {
            throw new ParseException(
                "Invalid value for type Number: " + v);
        }
    }

    boolean lt(Value v) { return (value < ((NumberVal)v).intValue()); }
    boolean gt(Value v) { return (value > ((NumberVal)v).intValue()); }

    public void setVarNum(int varNum) { this.varNum = varNum; }
    public int getVarNum() { return varNum; }

    public String toString() { return String.valueOf(value); }

}