/*
    Copyright (C) 1999 by Joe Pasqua <joe@NoRoomAtTheInn.org>.
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.


    THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

import java.util.*;

class SymTab {
    Vector  tables = new Vector(1);

    SymTab() {
        tables.addElement(new Hashtable());
    }

    Symbol get(String key) {
        for (int i = 0; i < tables.size(); i++) {
            Symbol s = (Symbol)((Hashtable)tables.elementAt(i)).get(key);
            if (s != null)
                return s;
        }
        return null;
    }

    void put(String key, Symbol s) {
        ((Hashtable)tables.lastElement()).put(key, s);
    }

    void add(Symbol s) { put(s.name, s); }

    void addConst(String name, String type, String value)
            throws ParseException
    {
        add(new Symbol.Constant(name, Type.get(type), new BasicVal(value)));
    }

    void addVar(String name, String type)
            throws ParseException
    {
        add(new Symbol.Variable(name, Type.get(type)));
    }

    void push() {
        tables.addElement(new Hashtable());
    }

    void pop() {
        tables.setSize(tables.size()-1);
        if (tables.size() == 0)
            throw new RuntimeException("Popped off root symbol table!");
    }

    Enumeration keys() {
        Vector  enums = new Vector(tables.size());
        for (int i = 0; i < tables.size(); i++)
            enums.addElement( ((Hashtable)tables.elementAt(i)).keys() );
        return new MultiEnum(enums);
    }
}